#include <arduino.h>
#include <Wire.h>
#include <arduino.h>
#include "BMP280.h"

#define BMP280_ADDR     0x77
#define BMP280_ID       0xD0
#define BMP280_RESET    0xE0
#define BMP280_CONTROL  0xF4
#define BMP280_PRESSURE 0xF7
#define BMP280_TEMP     0xFA

#define BMP280_DIG_T1    0x88
#define BMP280_DIG_T2    0x8A
#define BMP280_DIG_T3    0x8C
#define BMP280_DIG_P1    0x8E
#define BMP280_DIG_P2    0x90
#define BMP280_DIG_P3    0x92
#define BMP280_DIG_P4    0x94
#define BMP280_DIG_P5    0x96
#define BMP280_DIG_P6    0x98
#define BMP280_DIG_P7    0x9A
#define BMP280_DIG_P8    0x9C
#define BMP280_DIG_P9    0x9E

#define BMP280_T1 bmp280DIG[0]
#define BMP280_T2 bmp280DIG[1]
#define BMP280_T3 bmp280DIG[2]
#define BMP280_P1 bmp280DIG[3]
#define BMP280_P2 bmp280DIG[4]
#define BMP280_P3 bmp280DIG[5]
#define BMP280_P4 bmp280DIG[6]
#define BMP280_P5 bmp280DIG[7]
#define BMP280_P6 bmp280DIG[8]
#define BMP280_P7 bmp280DIG[9]
#define BMP280_P8 bmp280DIG[10]
#define BMP280_P9 bmp280DIG[11]

bool BMP280::init() {
  Wire.begin();

  t_fine = 0 ;
  temperature = 0 ; // 温度 単位 1/00℃
  presser = 0 ;     // 気圧 単位 Pa

  // BMP280リセット
  sendBMP280(BMP280_RESET,0xB6) ;
  delay(4) ;  // 通常軌道に2m秒を要するので、多めに4m秒待ってみる

  if (readBMP32b8(BMP280_ID) != 0x58) { // デバイスIDチェック
    return false ;
  }

  // 補正データ読み出し
  BMP280_T1 = readBMP32b16LE(BMP280_DIG_T1) ;
  BMP280_T2 = readBMP32b16LE(BMP280_DIG_T2) ;
  BMP280_T3 = readBMP32b16LE(BMP280_DIG_T3) ;

  BMP280_P1 = readBMP32b16LE(BMP280_DIG_P1) ;
  BMP280_P2 = readBMP32b16LE(BMP280_DIG_P2) ;
  BMP280_P3 = readBMP32b16LE(BMP280_DIG_P3) ;
  BMP280_P4 = readBMP32b16LE(BMP280_DIG_P4) ;
  BMP280_P5 = readBMP32b16LE(BMP280_DIG_P5) ;
  BMP280_P6 = readBMP32b16LE(BMP280_DIG_P6) ;
  BMP280_P7 = readBMP32b16LE(BMP280_DIG_P7) ;
  BMP280_P8 = readBMP32b16LE(BMP280_DIG_P8) ;
  BMP280_P9 = readBMP32b16LE(BMP280_DIG_P9) ;

  // 動作モード設定
  sendBMP280(BMP280_CONTROL,0b00111111) ; // 001:温度の解像度　111：気圧の解像度　11:動作モード(ノーマルモード)

  return true ;
}

// BMP280に1バイトコマンドを送信
void BMP280::sendBMP280(uint8_t addr , uint8_t command) {
  Wire.beginTransmission(BMP280_ADDR);
  Wire.write(addr);
  Wire.write(command);
  Wire.endTransmission();
}

// センサー読み取り
uint32_t BMP280::readBMP32b20(uint8_t addr) {
  uint32_t sensorData = 0 ;
  uint8_t readData ;

  // Read Address Set
  Wire.beginTransmission(BMP280_ADDR);
  Wire.write(addr);
  Wire.endTransmission();
  
  // 3Byte Read
  Wire.requestFrom(BMP280_ADDR, 3);
  while(Wire.available()<3);
  readData = Wire.read();
  sensorData = readData ;
  readData = Wire.read();
  sensorData = (sensorData << 8) | readData ;
  readData = Wire.read();
  sensorData = (sensorData << 8) | readData ;
  sensorData >>= 4 ;

  return sensorData ;
}

// 補正値読み取り
uint16_t BMP280::readBMP32b16LE(uint8_t addr) {
  uint16_t readData ;
  // Read Address Set
  Wire.beginTransmission(BMP280_ADDR);
  Wire.write(addr);
  Wire.endTransmission();
  
  // 2Byte Read
  Wire.requestFrom(BMP280_ADDR, 2);
  while(Wire.available()<2);
  uint8_t d1 = Wire.read();
  uint8_t d2 = Wire.read();
  readData = d2 << 8 | d1 ;

  return readData ;
}

// Register読み取り
uint8_t BMP280::readBMP32b8(uint8_t addr) {
  uint8_t readData ;
  // Read Address Set
  Wire.beginTransmission(BMP280_ADDR);
  Wire.write(addr);
  Wire.endTransmission();
  
  // 1Byte Read
  Wire.requestFrom(BMP280_ADDR, 1);
  while(Wire.available()<1);
  readData = Wire.read();

  return readData ;
}

// 温度読み取り
int32_t BMP280::readBMP32Tmp() {
  uint32_t temData = readBMP32b20(BMP280_TEMP) ;

  // ----- DATA 補正 -----
  int32_t var1, var2;
  uint16_t t1 = BMP280_T1 ;
  int16_t t2 = (int16_t)BMP280_T2,t3=(int16_t)BMP280_T3 ;
  var1 = (((temData >> 3) - ((int32_t)(t1 << 1))) * ((int32_t)t2)) >> 11;
  var2 = (((((temData >> 4) - ((int32_t)t1)) * ((temData >> 4) - ((int32_t)t1))) >> 12) * ((int32_t)t3)) >> 14;

  t_fine = var1 + var2;
  temData = (t_fine * 5 + 128) >> 8;

  return temData ;
}

// 気圧読み取り
int32_t BMP280::readBMP32Pre() {
  readBMP32Tmp();
  uint32_t preData = readBMP32b20(BMP280_PRESSURE) ;

  // ----- DATA 補正 -----
  int64_t var1, var2 , p;
  uint16_t p1 = BMP280_P1 ;
  int16_t p2 = (int16_t)BMP280_P2,p3=(int16_t)BMP280_P3 ;
  int16_t p4 = (int16_t)BMP280_P4,p5=(int16_t)BMP280_P5 ;
  int16_t p6 = (int16_t)BMP280_P6,p7=(int16_t)BMP280_P7 ;
  int16_t p8 = (int16_t)BMP280_P8,p9=(int16_t)BMP280_P9 ;

  var1 = ((int64_t)t_fine) - 128000;
  var2 = var1 * var1 * (int64_t)p6;
  var2 = var2 + ((var1*(int64_t)p5)<<17);
  var2 = var2 + (((int64_t)p4)<<35);
  var1 = ((var1 * var1 * (int64_t)p3)>>8) + ((var1 * (int64_t)p2)<<12);
  var1 = (((((int64_t)1)<<47)+var1))*((int64_t)p1)>>33;
  if (var1 == 0) {
    return 0; // avoid exception caused by division by zero
  }
  p = 1048576-preData;
  p = (((p<<31)-var2)*3125)/var1;
  var1 = (((int64_t)p9) * (p>>13) * (p>>13)) >> 25;
  var2 = (((int64_t)p8) * p) >> 19;
  p = ((p + var1 + var2) >> 8) + (((int64_t)p7)<<4);
  preData = (uint32_t)p /256 ;

  return preData ;
}
