/*
 * SSD1306/SSD1315に温度・気圧計を表示
 * 
 */

#include "DISPGraph.h"
#define GRAPHLEN 80

static uint8_t graphBitPattern[] = {
  0x80,0xC0,0xE0,0xF0,0xF8,0xFC,0xFE,0xFF
} ;

// ----- 初期化 -----
void DISPGraph::init() {
  disp7seg.init();
  initGraphData() ;
}

// ---------------------
// ----- グラフ表示 -----
// ---------------------

// ----- グラフData初期化 ----
void DISPGraph::initGraphData() {
  graphDataNum = 0 ;
  memset(graphData,0,sizeof(graphData) ) ;
}

// ----- グラフDataSet ----
void DISPGraph::addGraphData(int32_t temperature , int32_t presser ) {
  if (graphDataNum == 79) {
    graphData[graphDataNum].temperature = temperature / 100 ; 
    graphData[graphDataNum].presser = presser / 100 ; 
    memcpy(graphData,&graphData[1],sizeof(struct graphData_s) * 79) ;
  } else {
    graphData[graphDataNum].temperature = temperature / 100 ; 
    graphData[graphDataNum].presser = presser / 100 ; 
    graphDataNum ++ ;
  }
}

// ----- グラフData表示 ----
void DISPGraph::dispGraphData() {
  uint8_t graphBuff[4][85] ;
  // ----- 
  disp7seg.disp7SEG(  0 , 0 , 11 ) ;
  disp7seg.disp7SEG(  0 , 4 , 16 ) ;
  
  int i,d,p,b,n,nd ;

  n = (graphDataNum > 0 ) ? graphDataNum -1 : 0 ; 
  int tip = graphData[n].temperature ;
  int tipMin = tip ; 
  int tipMax = tip ;

  int pre = graphData[n].presser ;
  int preMin = pre ; 
  int preMax = pre ;

  for (i = 0 ; i < graphDataNum ; i++) {
    if (tipMin > graphData[i].temperature ) { tipMin = graphData[i].temperature ; }
    if (tipMax < graphData[i].temperature ) { tipMax = graphData[i].temperature ; }

    if (preMin > graphData[i].presser) { preMin = graphData[i].presser ; }
    if (preMax < graphData[i].presser) { preMax = graphData[i].presser ; }
  }
  // ----- 気温 -----
  memset(graphBuff,0,sizeof(graphBuff)) ;

  if (tipMax - tipMin <= 10) {
    tipMax = (tipMax+tipMin) / 2  + 5; 
    tipMin = tipMax - 10 ;
  }
  disp7seg.disp7GraphFont(  27 , 0 , ('0' + tipMax / 10) ) ;
  disp7seg.disp7GraphFont(  32 , 0 , ('0' + tipMax % 10) ) ;

  disp7seg.disp7GraphFont(  27 , 3 , ('0' + tipMin / 10) ) ;
  disp7seg.disp7GraphFont(  32 , 3 , ('0' + tipMin % 10) ) ;

  graphBuff[0][4] = 0xFF ;
  graphBuff[1][4] = 0xFF ;
  graphBuff[2][4] = 0xFF ;
  graphBuff[3][4] = 0xFF ;

  // --- tipMin
  for (i = 0 ; i < 85 ; i++) {
    graphBuff[0][i] ^= 0x08 ;
  }

  // --- tipMax
  for (i = 0 ; i < 85 ; i++) {
    graphBuff[3][i] ^= 0x10 ;
  }

  int tipRange = tipMax - tipMin ;
  for (i = 0 ; i < graphDataNum ; i++) {
    n = graphData[i].temperature ;
    if (tipMin <= n && n <= tipMax) {
      d = ((n-tipMin)*10.+nd) / tipRange * 2.4 + 4 ;
      p = 3 - ((d / 8) % 4) ;
      b = d % 8 ;
      // ---- BitPattern設定
      uint8_t bitData = graphBitPattern[b] ;
      if (p == 3) { bitData &= 0x0F ; }
      if (p <= 2) { graphBuff[3][i+5] = 0x0F ; }
      if (p <= 1) { graphBuff[2][i+5] = 0xFF ; }
      if (p == 0) { graphBuff[1][i+5] = 0xFF ; bitData &= 0xF0 ;}
      graphBuff[p][i+5] ^= bitData ;
//      graphBuff[p][i+5] ^= 0x80 >> b ;
    }
  }
  disp7seg.oledBitMap(40,0,85 ,4,(uint8_t *)&graphBuff[0][0]) ;


  // -- 湿度
  memset(graphBuff,0,sizeof(graphBuff)) ;
  if (preMax - preMin <= 10) {
    preMax = (preMax+preMin) / 2  + 5; 
    preMin = preMax - 10 ;
  }

  disp7seg.disp7GraphFont(  17 , 4 , ('0' + preMax / 1000) ) ;
  disp7seg.disp7GraphFont(  22 , 4 , ('0' + (preMax % 1000) / 100) ) ;
  disp7seg.disp7GraphFont(  27 , 4 , ('0' + (preMax % 100) / 10)) ;
  disp7seg.disp7GraphFont(  32 , 4 , ('0' + preMax % 10 ) ) ;

  disp7seg.disp7GraphFont(  17 , 7 , ('0' + preMin / 1000) ) ;
  disp7seg.disp7GraphFont(  22 , 7 , ('0' + (preMin % 1000) / 100) ) ;
  disp7seg.disp7GraphFont(  27 , 7 , ('0' + (preMin % 100) / 10) ) ;
  disp7seg.disp7GraphFont(  32 , 7 , ('0' + preMin % 10) ) ;

  graphBuff[0][4] = 0xFF ;
  graphBuff[1][4] = 0xFF ;
  graphBuff[2][4] = 0xFF ;
  graphBuff[3][4] = 0xFF ;

  // --- tipMin
  for (i = 0 ; i < 85 ; i++) {
    graphBuff[0][i] ^= 0x08 ;
  }

  // --- tipMax
  for (i = 0 ; i < 85 ; i++) {
    graphBuff[3][i] ^= 0x10 ;
  }

  int preRange = preMax - preMin ;
  for (i = 0 ; i < graphDataNum ; i++) {
    n = graphData[i].presser ;
    if (preMin <= n && n <= preMax) {
      d = ((n-preMin)*10.) / preRange * 2.4 + 4 ;
      p = 3 - ((d / 8) % 4) ;
      b = d % 8 ;
      // ---- BitPattern設定
      uint8_t bitData = graphBitPattern[b] ;
      if (p == 3) { bitData &= 0x0F ; }
      if (p <= 2) { graphBuff[3][i+5] = 0x0F ; }
      if (p <= 1) { graphBuff[2][i+5] = 0xFF ; }
      if (p == 0) { graphBuff[1][i+5] = 0xFF ;  bitData &= 0xF0 ; }
      graphBuff[p][i+5] ^= bitData ;
//      graphBuff[p][i+5] ^= 0x80 >> b ;
    }
  }
  disp7seg.oledBitMap(40,4,85,4,(uint8_t *)&graphBuff[0][0]) ;
  
  
  disp7seg.dispSwitch(true) ;
}

// ---------------------------------
// ----- 7セグメントLEDッポイ表示 -----
// ---------------------------------
/**
 * int32_t temperature - 温度
 * int32_t presser - 気圧
 */
void DISPGraph::disp7segLED(int32_t temperature , int32_t presser) {
  // --- 7セグメントLEDっぽい表示 ---
  int t1 = abs(temperature) / 1000 ;
  int t2 = (abs(temperature) % 1000) / 100 ;
  int td = (abs(temperature) % 100) / 10 ;
  if (temperature < 0) { t1 = -1; }
  
  int p1 = presser / 100000 ;
  int p2 = (presser % 100000) / 10000 ;
  int p3 = (presser % 10000) / 1000  ;
  int p4 = (presser % 1000) / 100 ;

  // ----- SSD1306に表示 -----
  disp7seg.disp7SEG(  0 , 0 , 11 ) ;
  if (temperature < 0) {
    disp7seg.disp7SEG( 20 , 0 , 18 ) ;
  } else {
    disp7seg.disp7SEG( 20 , 0 , t1 ) ;
  }
  disp7seg.disp7SEG( 40 , 0 , t2 ) ;
  disp7seg.disp7SEG( 56 , 0 , 10 ) ;
  disp7seg.disp7SEG( 72 , 0 , td ) ;
  disp7seg.disp7SEG( 92 , 0 , 12 ) ;

  disp7seg.disp7SEG(  0 , 4 , 16 ) ;
  disp7seg.disp7SEG( 20 , 4 , p1 ) ;
  disp7seg.disp7SEG( 40 , 4 , p2 ) ;
  disp7seg.disp7SEG( 60 , 4 , p3 ) ;
  disp7seg.disp7SEG( 80 , 4 , p4 ) ;
  disp7seg.disp7SEG( 100, 4 , 17 ) ;

  disp7seg.dispSwitch(true) ;
}
